import data, math, random, pixelperfect

import pygame
from pygame.locals import *

SCORE = 0
LEVEL = 1
BASE_HEALTH = 50

class Gun(pygame.sprite.Sprite):

    def __init__(self):

        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = data.load_image("gun.png")
        self._image = self.image
        self.rect = self.image.get_rect(center = (240, 460))
        self.angle = 0
        self.timer = 0
        self.sound = data.load_sound("shoot.ogg")

    def rotate(self):
        _center = self.rect.center
        self.image = pygame.transform.rotate(self._image, self.angle)
        self.rect = self.image.get_rect(center = _center)

    def update(self):
        self.rotate()
        self.timer += 1
        key = pygame.key.get_pressed()
        if self.angle < 0:
            self.angle = 360
        if self.angle > 360:
            self.angle = 0
        if key[K_LEFT]:
            self.angle += 2
        if key[K_RIGHT]:
            self.angle -= 2
        if key[K_SPACE]:
            if self.timer >= 8:
                Shot(self.rect.center, self.angle)
                self.timer = 0
                self.sound.play()
        if self.angle > 90 and self.angle < 180:
            self.angle = 90
        if self.angle < 270 and self.angle > 180:
            self.angle = 270

class Base(pygame.sprite.Sprite):

    def __init__(self):

        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = data.load_image("base.png")
        self.rect = self.image.get_rect(center = (240, 450))

class Shot(pygame.sprite.Sprite):

    def __init__(self, pos, angle):

        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = data.load_image("bullet.png")
        self.rect = self.image.get_rect(center = pos)
        self.x = pos[0]
        self.y = pos[1]
        self.angle = angle
        self.x -= 40*math.sin(math.radians(self.angle))
        self.y -= 40*math.cos(math.radians(self.angle))
        self.rect.center = self.x, self.y

    def update(self):
        self.x -= 8*math.sin(math.radians(self.angle))
        self.y -= 8*math.cos(math.radians(self.angle))
        self.rect.center = self.x, self.y
        screenrect = Rect(0, 0, 640, 480)
        if not screenrect.contains(self.rect):
            self.kill()

class Helicopter(pygame.sprite.Sprite):

    def __init__(self):

        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = data.load_image("helicopter.png")
        self.rect = self.image.get_rect(midright = (0, random.randrange(64, 128)))
        self.dir = random.choice([1, -1])
        if self.dir < 0:
            self.rect.left = 480
            self.image = pygame.transform.flip(self.image, 1, 0)

    def update(self):
        self.rect.move_ip(2*self.dir, 0)
        if self.rect.left > 490:
            self.kill()
        if self.rect.right < -10:
            self.kill()

class Bomb(pygame.sprite.Sprite):

    def __init__(self, pos):

        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = data.load_image("bomb.png")
        self.rect = self.image.get_rect(midtop = pos)

    def update(self):
        self.rect.move_ip(0, 3)
        if self.rect.bottom >= 470:
            global BASE_HEALTH
            BASE_HEALTH -= 1 
            Explosion(self.rect.center)
            self.kill()

class Explosion(pygame.sprite.Sprite):

    def __init__(self, pos):

        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = pygame.Surface((64, 64))
        self.image.set_colorkey((0, 0, 0), RLEACCEL)
        self.rect = self.image.get_rect(center = pos)
        self.scale = 20
        self.alpha = 255

    def update(self):
        self.scale += 4
        if self.scale >= 64:
            self.scale = 64
            self.alpha -= 10
        if self.alpha <= 0:
            self.kill()
        pygame.draw.ellipse(self.image, (255, 230, 0), (32-self.scale/2, 32-self.scale/2, self.scale, self.scale))
        self.image.set_alpha(self.alpha)

def menu(screen):
    background = data.load_image("menu.png")

    while 1:

        for e in pygame.event.get():
            if e.type == QUIT:
                pygame.quit()
                return
            if e.type == KEYDOWN:
                if e.key == K_ESCAPE:
                    pygame.quit()
                    return
                if e.key == K_SPACE:
                    game(screen)

        screen.blit(background, (0, 0))
        pygame.display.flip()


def game(screen):

    all = pygame.sprite.OrderedUpdates()
    helicopters = pygame.sprite.Group()
    shots = pygame.sprite.Group()
    bombs = pygame.sprite.Group()

    Gun.containers = all
    Base.containers = all
    Shot.containers = all, shots
    Helicopter.containers = all, helicopters
    Bomb.containers = all, bombs
    Explosion.containers = all

    clock = pygame.time.Clock()
    background = data.load_image("background.png")
    font = pygame.font.Font(data.filepath("Vera.ttf"), 16)
    font2 = pygame.font.Font(data.filepath("Vera.ttf"), 32)
    gun = Gun()
    base = Base()
    Helicopter()
    boom = data.load_sound("boom.ogg")

    global SCORE, LEVEL, BASE_HEALTH
    SCORE = 0
    LEVEL = 1
    BASE_HEALTH = 50

    while 1:

        clock.tick(60)
        all.update()

        for e in pygame.event.get():
            if e.type == QUIT:
                pygame.quit()
                return
            if e.type == KEYDOWN:
                if e.key == K_ESCAPE:
                    return

        for h in pixelperfect.groupcollide_pp(helicopters, shots, 1, 1):
            Explosion(h.rect.center)
            SCORE += 75
            boom.play()
            if not helicopters:
                LEVEL += 1
        for b in pixelperfect.groupcollide_pp(bombs, shots, 1, 1):
            Explosion(b.rect.center)
            SCORE += 25
            boom.play()

        if not random.randrange(50) and len(helicopters) < LEVEL and BASE_HEALTH > 0:
            Helicopter()
        for heli in helicopters:
            if not random.randrange(150-LEVEL*10) and BASE_HEALTH > 0:
                Bomb(heli.rect.midbottom)

        if BASE_HEALTH <= 0:
            BASE_HEALTH = 0
            if gun.alive(): Explosion(gun.rect.center)
            if base.alive(): Explosion(base.rect.center)
            gun.kill()
            base.kill()
            Explosion((random.randrange(480), random.randrange(460, 480)))
            if not random.randrange(10):
                boom.play()

        screen.blit(background, (0, 0))
        all.draw(screen)
        ren = font.render("Score: %d" % SCORE, 1, (0, 0, 0))
        screen.blit(ren, (151, 11))
        ren = font.render("Score: %d" % SCORE, 1, (255, 255, 255))
        screen.blit(ren, (150, 10))
        ren = font.render("Level: %d" % LEVEL, 1, (0, 0, 0))
        screen.blit(ren, (301, 11))
        ren = font.render("Level: %d" % LEVEL, 1, (255, 255, 255))
        screen.blit(ren, (300, 10))
        pygame.draw.rect(screen, (255, 0, 0), (10, 15, BASE_HEALTH*2, 10))
        pygame.draw.rect(screen, (0, 0, 0), (10, 15, 100, 10), 1)
        if BASE_HEALTH <= 0:
            ren = font2.render("Game Over!", 1, (0, 0, 0))
            screen.blit(ren, (242-ren.get_width()/2, 242-ren.get_height()/2))
            ren = font2.render("Game Over!", 1, (255, 255, 255))
            screen.blit(ren, (240-ren.get_width()/2, 240-ren.get_height()/2))
        pygame.display.flip()


def main():

    pygame.init()

    pygame.display.set_caption("Helicopters - PyWeek Number 6 Warmup - PyMike 2008")
    screen = pygame.display.set_mode((480, 480))
    pygame.mixer.music.load(data.filepath("loop.ogg"))
    pygame.mixer.music.play(-1)
    menu(screen)
